-- AlwaysLFG 1.0
--
-- Automatically adds the user to the LFG channel on login and when
-- leaving a group, without interfering with actual use of the LFG tool.
--
-- Send suggestions, comments, and bugs to merial.kilrogg@gmail.com.

local options = {
  type = 'group',
  args = {
    join = {
      type = "execute",
      desc = "Join LFG channel now.",
      name = "Join LFG",
      func = "JoinLFG",
    },
    auto = {
      type = "toggle",
      desc = "Automatically join LFG channel when leaving groups.",
      name = "Join on Leave",
      get = "IsEnabled",
      set = "ToggleEnable",
    },
    verbose = {
      type = "toggle",
      desc = "Show console messages when resetting LFG parameters.",
      name = "Verbose",
      get = "IsVerbose",
      set = "ToggleVerbose",
    },
  },
}

AlwaysLFG = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0")
AlwaysLFG:RegisterChatCommand({"/alfg", "/alwayslfg"}, options)

AlwaysLFG.enabled = true
AlwaysLFG.verbose = false
AlwaysLFG.inGroup = false
AlwaysLFG.initialJoin = false

-- Get verbose flag.
function AlwaysLFG:IsVerbose()
  return self.verbose;
end

-- Update verbose flag.
function AlwaysLFG:ToggleVerbose()
  self.verbose = not self.verbose;
end

-- Print verbose messages.
function AlwaysLFG:DebugPrint(msg)
  if self.verbose then
    self:Print(msg);
  end
end

-- Get enabled flag.
function AlwaysLFG:IsEnabled()
  return self.enabled;
end

-- Enable AlwaysLFG.
function AlwaysLFG:ToggleEnable()
  self.enabled = not self.enabled;
end

-- Check whether we're already in LFG/LFM.
function AlwaysLFG:InLFG()
  local _, _, _, _, _, _, _, _, _, queued, lfgStatus, lfmStatus =
    GetLookingForGroup();
  return (lfgStatus or lfmStatus);
end

-- Add ourselves to LFM if we're not already LFG/LFM.
function AlwaysLFG:JoinLFG()
  if not self:InLFG() then
    ClearLookingForMore();
    SetLFMType(5);
    SetLookingForMore(5, 1);
    SetLFGComment("Listening to LFG channel using AlwaysLFG.");
    self:DebugPrint("Modified LFG settings.");
  end
end

-- Set our hooks.
function AlwaysLFG:OnEnable()
  self:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE");
  self:RegisterEvent("PARTY_MEMBERS_CHANGED");
  self:DebugPrint("Loaded.");
end

-- Join LFG the first time we join general chat (i.e., on login).
function AlwaysLFG:CHAT_MSG_CHANNEL_NOTICE(a1, a2, a3, a4, a5, a6, a7, a8, a9)
  if not self.initialJoin and a1 == "YOU_JOINED" and a7 == 1 then
    self:JoinLFG();
    self.initialJoin = true;
  end
end

-- Join LFG whenever we leave a group.
function AlwaysLFG:PARTY_MEMBERS_CHANGED()
  local nowInGroup =
    (GetRealNumPartyMembers() > 0 or GetRealNumRaidMembers() > 0);
  if self.inGroup and not nowInGroup and self.enabled then
    self:JoinLFG();
  end
  self.inGroup = nowInGroup;
end
